Package: apeglm
Title: Approximate posterior estimation for GLM coefficients
Version: 1.33.0
Authors@R: c(
    person("Anqi", "Zhu", email = "anqizhu@live.unc.edu", role = c("aut","cre")),
    person("Joshua", "Zitovsky", role = "ctb"),
    person("Joseph", "Ibrahim", role = "aut"),
    person("Michael", "Love", role = "aut"))
Maintainer: Anqi Zhu <anqizhu@live.unc.edu>
Description: apeglm provides Bayesian shrinkage estimators for effect sizes for a variety of GLM models, using approximation of the posterior for individual coefficients.
VignetteBuilder: knitr, rmarkdown
Imports: emdbook, SummarizedExperiment, GenomicRanges, methods, stats,
        utils, Rcpp
Suggests: DESeq2, airway, knitr, rmarkdown, testthat
LinkingTo: Rcpp, RcppEigen, RcppNumerical
License: GPL-2
Encoding: UTF-8
biocViews: ImmunoOncology, Sequencing, RNASeq, DifferentialExpression,
        GeneExpression, Bayesian
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/apeglm
git_branch: devel
git_last_commit: 3ff43e2
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: yes
Packaged: 2025-10-31 20:30:56 UTC; biocbuild
Author: Anqi Zhu [aut, cre],
  Joshua Zitovsky [ctb],
  Joseph Ibrahim [aut],
  Michael Love [aut]
