# Generated by roxygen2: do not edit by hand

export(apeglm)
export(bbEstDisp)
export(logLikNB)
export(svalue)
importFrom(GenomicRanges,"mcols<-")
importFrom(Rcpp,sourceCpp)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,rowRanges)
importFrom(emdbook,dbetabinom)
importFrom(methods,is)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,dt)
importFrom(stats,optim)
importFrom(stats,optimHess)
importFrom(stats,optimize)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(apeglm)
