# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

betabinGLM <- function(x, Y, sizes, thetas, weights, sigma2, S2, no_shrink, shrink, init, cnst, tol, lbd, ubd) {
    .Call('_apeglm_betabinGLM', PACKAGE = 'apeglm', x, Y, sizes, thetas, weights, sigma2, S2, no_shrink, shrink, init, cnst, tol, lbd, ubd)
}

nbinomGLM <- function(x, Y, size, weights, offset, sigma2, S2, no_shrink, shrink, init, cnst) {
    .Call('_apeglm_nbinomGLM', PACKAGE = 'apeglm', x, Y, size, weights, offset, sigma2, S2, no_shrink, shrink, init, cnst)
}

